# Todo
# Have the vault + mkdocs folder
# Use the metadata from YAFPA aka : folder
# Add css for img flags
# Add customization in mkdocs yml (logo ; description)
# Admonition code block (as always)
# Add links in YML markdown
# Create folder when category / folder doesn't exist ;)
